import os
from colorama import Fore
from acit1515.tests.test import Test
from pathlib import Path

class Is_Valid_Transcript(Test):
    def __init__(self, submission):
        self.name = f'{__class__.__name__.lower()}()'
        self.submission = submission

    def test1_returns(self):
        returns = True

        try:
            updated = self.submission.is_valid_transcript({})
        except Exception as e:
            returns = False
        else:
            if updated == None:
               returns = False 
        
        if not returns:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-4 marks)" + Fore.RESET)
            return { 'deduction': 4, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_type(self):
        correct = {
            "firstname": "Chris",
            "lastname": "Harris",
            "grades": [
                {"course": "ACIT 1515", "mark": 90},
                {"course": "ACIT 1620", "mark": 100}
            ]
        }
        incorrect = {}
        
        if type(self.submission.is_valid_transcript(correct)) != bool or type(self.submission.is_valid_transcript(incorrect)) != bool:
            print(Fore.RED + f"FAIL: {self.name} does not return a list of Path objects (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test3_hardcoded(self):
        correct = correct = {
            "firstname": "Chris",
            "lastname": "Harris",
            "grades": [
                {"course": "ACIT 1515", "mark": 90},
                {"course": "ACIT 1620", "mark": 100}
            ]
        }
        incorrect1 = {}
        incorrect2 = {
            "firstname": "Jeremy",
            "lastname": "Holman"
        }
        transcripts = [correct, incorrect1, incorrect2]

        if all(self.submission.is_valid_transcript(t) for t in transcripts) or all(not self.submission.is_valid_transcript(t) for t in transcripts):
            print(Fore.RED + f"FAIL: {self.name} appears to be returning a hardcoded value (-2 marks)" + Fore.RESET)
            return { 'deduction': 2, 'continue': False }
    
        return { 'deduction': 0, 'continue': True }
    
    def test4_valid(self):
        correct = correct = {
            "firstname": "Chris",
            "lastname": "Harris",
            "grades": [
                {"course": "ACIT 1515", "mark": 90},
                {"course": "ACIT 1620", "mark": 100}
            ]
        }
        incorrect = {
            "first": "Chris",
            "lastname": 10,
            "grades": (
                {"course": "ACIT 1515", "mark": 90},
                {"course": "ACIT 1620", "mark": 100}
            )
        }

        if not self.submission.is_valid_transcript(correct) or self.submission.is_valid_transcript(incorrect):
            print(Fore.RED + f"FAIL: {self.name} does not return a valid list of Path objects (-1 mark)" + Fore.RESET)
            return { 'deduction': 1, 'continue': False }

        print(Fore.GREEN + f"PASS: {self.name} returned a valid list of Path objects (4 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }


import os
from colorama import Fore
from acit1515.tests.test import Test
from pathlib import Path

class Get_File_List(Test):
    def __init__(self, submission):
        self.name = f'{__class__.__name__.lower()}()'
        self.submission = submission

    def test1_returns(self):
        returns = True

        try:
            updated = self.submission.get_file_list('./transcripts')
        except Exception as e:
            returns = False
        else:
            if updated == None:
               returns = False 
        
        if not returns:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-4 marks)" + Fore.RESET)
            return { 'deduction': 4, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_type(self):
        correct = self.submission.get_file_list('./transcripts')
        incorrect = self.submission.get_file_list('./someotherdirectory')
        
        if type(correct) != list or (type(correct) == list and not all(isinstance(p, Path) for p in correct)):
            print(Fore.RED + f"FAIL: {self.name} does not return a list of Path objects (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False } 

        if incorrect != False:
            print(Fore.RED + f"FAIL: {self.name} does not return False for an invalid directory (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test3_hardcoded(self):
        current_dir = "/".join(os.path.realpath(__file__).split('\\')[:-1])
        transcripts_path = Path(current_dir + "/transcripts")
        mock_path = Path(current_dir + '/mock')
        files1 = self.submission.get_file_list(transcripts_path.resolve())
        files2 = self.submission.get_file_list(mock_path.resolve())

        if files1 == files2:
            print(Fore.RED + f"FAIL: {self.name} appears to be returning a hardcoded value (-2 marks)" + Fore.RESET)
            return { 'deduction': 2, 'continue': False }
    
        return { 'deduction': 0, 'continue': True }
    
    def test4_valid(self):
        valid = True

        current_dir = "/".join(os.path.realpath(__file__).split('\\')[:-1])
        transcripts_path = Path(current_dir + "/transcripts")
        mock_path = Path(current_dir + '/mock')
        
        files1 = [p.name for p in self.submission.get_file_list(transcripts_path)]
        files2 = [p.name for p in self.submission.get_file_list(mock_path)]

        if (
            "Test-User.json" not in files1 or
            "User-Test.json" not in files1
        ) or (
            "mock.txt" not in files2
        ):
            print(Fore.RED + f"FAIL: {self.name} does not return a valid list of Path objects (-1 mark)" + Fore.RESET)
            return { 'deduction': 1, 'continue': False }

        print(Fore.GREEN + f"PASS: {self.name} returned a valid list of Path objects (4 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }


from colorama import Fore
from acit1515.tests.test import Test

class Generate_Grades(Test):
    def __init__(self, submission):
        self.name = 'generate_grades()'
        self.submission = submission

    def test1_returns(self):
        test_grades = self.submission.generate_grades()
        if test_grades == None:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-5 marks)" + Fore.RESET)
            return { 'deduction': 5, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }

    def test2_list(self):
        test_grades = self.submission.generate_grades()
        if not isinstance(test_grades, list):
            print(Fore.RED + f"FAIL: {self.name} does not return a list (-4 marks)" + Fore.RESET)
            return { 'deduction': 4, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
        
    def test3_length(self):
        test_grades = self.submission.generate_grades()
        if len(test_grades) != 7:
            print(Fore.RED + f"FAIL: {self.name} did not produce a list of length 7 (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False } 

        return { 'deduction': 0, 'continue': True }
    
    def test4_hardcoded(self):
        test_grades = [self.submission.generate_grades(), self.submission.generate_grades(), self.submission.generate_grades()]  

        test_grades_set = set(self.submission.generate_grades())
        
        if (test_grades[0] == test_grades[1] and test_grades[0] == test_grades[2]) or len(test_grades_set) == 1:
            print(Fore.RED + f"FAIL: {self.name} appears to be returning hard coded values (-2 marks)" + Fore.RESET)
            return { 'deduction': 2, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test5_valid(self):
        test_grades = self.submission.generate_grades()

        for v in test_grades:
            if v < 0 or v > 100:
                print(Fore.RED + f"FAIL: {self.name} returned invalid values (-1 marks)" + Fore.RESET)
                return { 'deduction': 1, 'continue': False }

        print(Fore.GREEN + f"PASS: {self.name} produced a valid list (5 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }


import os
from colorama import Fore
from acit1515.tests.test import Test
from pathlib import Path

class Get_Dir(Test):
    def __init__(self, submission):
        self.name = f'{__class__.__name__.lower()}()'
        self.submission = submission

    def test1_returns(self):
        returns = True

        try:
            updated = self.submission.get_dir('./transcripts')
        except Exception as e:
            returns = False
        else:
            if updated == None:
               returns = False 
        
        if not returns:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-4 marks)" + Fore.RESET)
            return { 'deduction': 4, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_type(self):
        correct = self.submission.get_dir('./transcripts')
        incorrect = self.submission.get_dir('./someotherdirectory')
        
        if not isinstance(correct, Path):
            print(Fore.RED + f"FAIL: {self.name} does not return a Path object for valid directory (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False } 
        if incorrect != False:
            print(Fore.RED + f"FAIL: {self.name} does not return False for an invalid directory (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test3_hardcoded(self):
        current_dir = "/".join(os.path.realpath(__file__).split('\\')[:-1])
        transcripts_path = Path(current_dir + "/transcripts")
        mock_path = Path(current_dir + '/mock')
        dir1 = self.submission.get_dir(transcripts_path)
        dir2 = self.submission.get_dir(mock_path)

        dir1_files = dir1.glob('**/*')
        dir2_files = dir2.glob('**/*')

        if dir1_files == dir2_files:
            print(Fore.RED + f"FAIL: {self.name} appears to be returning a hardcoded value (-2 marks)" + Fore.RESET)
            return { 'deduction': 2, 'continue': False }
    
        return { 'deduction': 0, 'continue': True }
    
    def test4_valid(self):
        valid = True

        current_dir = "/".join(os.path.realpath(__file__).split('\\')[:-1])
        transcripts_path = Path(current_dir + "/transcripts")
        mock_path = Path(current_dir + '/mock')
        dir1 = self.submission.get_dir(transcripts_path)
        dir2 = self.submission.get_dir(mock_path)

        if not dir1.joinpath('Test-User.json').is_file() and not dir2.joinpath('mock.txt').is_file():
            print(Fore.RED + f"FAIL: {self.name} does not return the correct directory (-1 mark)" + Fore.RESET)
            return { 'deduction': 1, 'continue': False }

        print(Fore.GREEN + f"PASS: {self.name} returned a valid value (4 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }


import os
import random
import json
from colorama import Fore
from acit1515.tests.test import Test
from acit1515.helpers import Transcript
from pathlib import Path

class Change_Grade(Test):
    def __init__(self, submission):
        self.name = f'{__class__.__name__.lower()}()'
        self.submission = submission

    def test1_returns(self):
        returns = True
        
        valid_dir_string = "/".join(__file__.split('\\')[:-1]) + '/transcripts'
        valid_user = Transcript(
            "Test", 
            "User", 
            [
                {"course": "ACIT 1630", "mark": 74}, 
                {"course": "ACIT 1515", "mark": 65}, 
                {"course": "MATH 1310", "mark": 90}
            ]
        )
        valid_course = 0
        valid_mark = 99

        try:
            updated = self.submission.change_grade(valid_dir_string, valid_user, valid_course, valid_mark)
        except Exception as e:
            returns = False
        else:
            if updated != None:
                returns = False
        
        if not returns:
            print(Fore.RED + f"FAIL: {self.name} failed or incorrectly returned a value (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_hardcoded(self):
        returns = True
        
        valid_dir_string = "/".join(__file__.split('\\')[:-1]) + '/transcripts'
        valid_user = Transcript(
            "Test", 
            "User", 
            [
                {"course": "ACIT 1630", "mark": 74}, 
                {"course": "ACIT 1515", "mark": 65}, 
                {"course": "MATH 1310", "mark": 90}
            ]
        )
        another_valid_user = Transcript(
            "Test", 
            "User2", 
            [
                {"course": "ACIT 1630", "mark": 74}, 
                {"course": "ACIT 1515", "mark": 65}, 
                {"course": "MATH 1310", "mark": 90}
            ]
        )
        random_course = random.randint(0,2)
        random_mark = random.randint(0,100)
        another_random_course = random.randint(0,2)
        another_random_mark = random.randint(0,100)

        self.submission.change_grade(valid_dir_string, valid_user, random_course, random_mark)
        self.submission.change_grade(valid_dir_string, another_valid_user, another_random_course, another_random_mark)

        dir = Path("/".join(__file__.split('\\')[:-1]))
        valid_path = dir.joinpath('transcripts/Test-User.json')
        another_valid_path = dir.joinpath('transcripts/Test-User2.json')

        try:
            with valid_path.open() as file:
                data = json.load(file)

            with another_valid_path.open() as file:
                more_data = json.load(file)
        except:
            returns = False
        else:
            if data == more_data:
                returns = False

        if not returns:
            print(Fore.RED + f"FAIL: {self.name} appears to be returning a hardcoded value (-2 marks)" + Fore.RESET)
            return { 'deduction': 2, 'continue': False }
    
        return { 'deduction': 0, 'continue': True }
    
    def test3_valid(self):
        returns = True

        valid_dir_string = "/".join(__file__.split('\\')[:-1]) + '/transcripts'
        valid_user = Transcript(
            "Test", 
            "User", 
            [
                {"course": "ACIT 1630", "mark": 74}, 
                {"course": "ACIT 1515", "mark": 65}, 
                {"course": "MATH 1310", "mark": 90}
            ]
        )
        random_course = random.randint(0,2)
        random_mark = random.randint(0,100)

        self.submission.change_grade(valid_dir_string, valid_user, random_course, random_mark)

        dir = Path("/".join(__file__.split('\\')[:-1]))
        valid_path = dir.joinpath('transcripts/Test-User.json')

        try:
            with valid_path.open() as file:
                data = json.load(file)
        except:
            returns = False
        else:
            if not (
                valid_user.firstname == data["firstname"]
                and valid_user.lastname == data["lastname"]
                and valid_user.grades == data["grades"]
            ):
                returns = False
                
        if not returns:
            print(Fore.RED + f"FAIL: {self.name} does not successfully update a transcript file (-1 mark)" + Fore.RESET)
            return { 'deduction': 1, 'continue': False }

        print(Fore.GREEN + f"PASS: {self.name} returned a valid Transcript object (3 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }


class Transcript:
    def __init__(self, firstname: str, lastname: str, grades: dict[str, str]):
        self.firstname = firstname
        self.lastname = lastname
        self.grades = grades

    def print_grades(self, enumerated = False):
        obj_str = ''
        for i, grade in enumerate(self.grades):
            if enumerated:
                obj_str += f'{i + 1}. '
            obj_str += f'{grade["course"]}: {grade["mark"]}\n'
        return obj_str[:-1]

    def to_string(self, enumerated = False):
        obj_str = f'{self.firstname} {self.lastname}\n'
        obj_str += self.print_grades(enumerated)
        return obj_str
    



class Test():
    """
    MANIFESTO

    Assignments must consist of:
        - Simple functions with no side effects, no sys.exit()
        - Input and output confined to __main__ 

    Test functions must be organized in order of blocking requirements, where x is the maximum number of marks for the question, e.g.:
        First function tests if value is returned (x marks deducted)
        Second function tests if returned value is correct type (x - 1 marks deducted)
        Third function tests if returned value is valid (x - 2 marks deducted)
    """
    def run_tests(self):
        tests = [attr for attr in dir(self) if attr.startswith("test")]
        total_marks = len(tests)

        for test in tests:
            result = getattr(self, test)()
            total_marks -= result['deduction']
            if not result['continue']:
                break
        
        return (len(tests), len(tests) - total_marks)


from colorama import Fore
from acit1515.tests.test import Test

class Generate_Grade(Test):
    def __init__(self, submission):
        self.name = 'generate_grade()'
        self.submission = submission

    def test1_returns(self):
        is_none = self.submission.generate_grade()
        if is_none == None:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_hardcoded(self):
        grade_set = { self.submission.generate_grade(), self.submission.generate_grade(), self.submission.generate_grade(), self.submission.generate_grade() }
        if len(grade_set) == 1:
            print(Fore.RED + f"FAIL: {self.name} is returning a hard-coded value (-2 marks)" + Fore.RESET)
            return { 'deduction': 2, 'continue': False }
        
        return { 'deduction': 0, 'continue': True }
        
    def test3_valid(self):
        for i in range(10000):
            test_grade = self.submission.generate_grade()

            if not isinstance(test_grade, int) or test_grade < 0 or test_grade > 100:
                print(Fore.RED + f"FAIL: {self.name} produced an invalid value (-1 mark)" + Fore.RESET)
                return { 'deduction': 1, 'continue': False }
        
        print(Fore.GREEN + f"PASS: {self.name} produced a valid value (3 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }


from colorama import Fore
from acit1515.tests.test import Test

class Get_Menu(Test):
    def __init__(self, submission):
        self.name = 'get_menu()'
        self.submission = submission

    def test1_returns(self):
        is_none = self.submission.get_menu()
        if is_none == None:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-5 marks)" + Fore.RESET)
            return { 'deduction': 5, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_list(self):
        test_menu = self.submission.get_menu()
        if not isinstance(test_menu, list):
            print(Fore.RED + f"FAIL: {self.name} does not return a list (-4 marks)" + Fore.RESET)
            return { 'deduction': 4, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test3_length(self):
        test_menu = self.submission.get_menu()
        if len(test_menu) != 5:
            print(Fore.RED + f"FAIL: {self.name} did not produce a list of length 5 (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False } 

        return { 'deduction': 0, 'continue': True }
    
    def test4_tuples(self):
        test_menu = self.submission.get_menu()
        for value in test_menu:
            if not isinstance(value, tuple):
                print(Fore.RED + f"FAIL: {self.name} does not return a list of tuples (-2 marks)" + Fore.RESET)
                return { 'deduction': 2, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test5_enumerated(self):
        test_menu = self.submission.get_menu()
        for i in range(len(test_menu)):
            if not isinstance(test_menu[i][0], int) and test_menu[i][0] == i + 1:
                print(Fore.RED + f"FAIL: {self.name} does not return a correctly enumerated list (-1 mark)" + Fore.RESET)
                return { 'deduction': 1, 'continue': False } 
        
        print(Fore.GREEN + f"PASS: {self.name} produced a valid list (5 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }


from colorama import Fore
from acit1515.tests.test import Test

"""

    TODO: concatenate and log error strings?

"""

class Change_Grade(Test):
    def __init__(self, submission):
        self.name = 'change_grade()'
        self.submission = submission

    def test1_returns(self):
        returns = True

        try:
            updated = self.submission.change_grade([50, 50, 50, 50, 50, 50, 50], 1, 75)
        except Exception as e:
            returns = False
        else:
            if updated == None:
               returns = False 
            
        if not returns:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-6 marks)" + Fore.RESET)
            return { 'deduction': 6, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_list(self):
        updated = self.submission.change_grade([50, 50, 50, 50, 50, 50, 50], 1, 75)
        if not isinstance(updated, list):
            print(Fore.RED + f"FAIL: {self.name} does not return a list (-5 marks)" + Fore.RESET)
            return { 'deduction': 5, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test3_length(self):
        updated = self.submission.change_grade([50, 50, 50, 50, 50, 50, 50], 1, 75)
        if len(updated) != 7:
            print(Fore.RED + f"FAIL: {self.name} does not return a list of length 7 (-4 marks)" + Fore.RESET)
            return { 'deduction': 4, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test4_hardcoded(self):
        updated1 = self.submission.change_grade([50, 50, 50, 50, 50, 50, 50], 1, 75)
        updated2 = self.submission.change_grade([40, 50, 50, 50, 50, 50, 50], 1, 90)
        updated3 = self.submission.change_grade([30, 50, 50, 50, 50, 50, 50], 1, 45)
        
        if updated1[0] == updated2[0] and updated1[0] == updated3[0]:
            print(Fore.RED + f"FAIL: {self.name} appears to be returning hard-coded values (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test5_changed(self):
        """
        
            TODO: update instructions to ensure students are allowing the correct offset,
                i.e. a user running the program chooses '2' to change the second grade
                    which should update the list at position 1
        
        """
        try:
           updated = self.submission.change_grade([50, 0, 50, 50, 50, 50, 50], 7, 99) 
        except:
            print(Fore.RED + f"FAIL: {self.name} allows out of bounds list index (-2 marks)" + Fore.RESET)
            return { 'deduction': 2, 'continue': False } 
        
        updated = self.submission.change_grade([50, 0, 50, 50, 50, 50, 50], 2, 99)
        
        if updated[1] != 99:
            print(Fore.RED + f"FAIL: {self.name} does not correctly change grade (-2 marks)" + Fore.RESET)
            return { 'deduction': 2, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test6_valid(self):
        updated1 = self.submission.change_grade([50, 50, 50, 50, 50, 50, 50], 1, 101)
        updated2 = self.submission.change_grade([50, 50, 50, 50, 50, 50, 50], 1, -1)

        if updated1 == None or updated2 == None:
            print(Fore.RED + f"FAIL: {self.name} does not return a value for invalid data (-1 marks)" + Fore.RESET)
            return { 'deduction': 1, 'continue': False }

        if updated1[0] != 50 and updated2[0] != 50:
            print(Fore.RED + f"FAIL: {self.name} does not correctly ignore invalid grades (-1 marks)" + Fore.RESET)
            return { 'deduction': 1, 'continue': False }

        print(Fore.GREEN + f"PASS: {self.name} produced a valid list (5 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }


import importlib
import importlib.util
import os
import sys
from colorama import Fore
from . import extract

def run_test(test, input_file):
    try:
        """
        
        TODO: ensure path construction works on all operating systems and for all variants
        
        """
        path_parts = input_file.split('\\')
        mod = path_parts[-1]
        pkg = "/".join(path_parts[:-1])
        
        spec = importlib.util.spec_from_file_location(f"{pkg}.{mod}", input_file)
        submission = importlib.util.module_from_spec(spec)
        sys.modules[f"{pkg}.{mod}"] = submission
        spec.loader.exec_module(submission)
    except Exception as e:
        print(Fore.RED + f'FAIL: script could not be run' + Fore.RESET)
        print(e)
        print(f'Final mark: 0')
        sys.exit()
    else:
        results = []
        directory = "/".join(os.path.realpath(__file__).split('\\')[:-1])

        for filename in os.listdir(os.path.join(directory, f'tests/{test}')):
            if filename.startswith('test_'):
                module_name = f'acit1515.tests.{test}.{filename[:-3]}'
                module = importlib.import_module(module_name)
                class_name = filename[5:-3].title()
                results.append(getattr(module, class_name)(submission).run_tests())

        total_marks = sum([total for total, _ in results])
        deductions = sum([deduction for _, deduction in results])

        print(f'Final mark: {total_marks - deductions}/{total_marks}')

def run_tests(test_suite, target_directory):
    res = extract.extract_submissions(target_directory)

    if not res['ERROR_STATE']:
        for submission in res['output_path'].glob('*.py'):
            print(Fore.CYAN + f'\nRUNNING TESTS FOR .\\{submission}' + Fore.RESET)
            try:
                run_test(test_suite, f'.\\{submission}')
                print()
            except Exception as e:
                print(Fore.YELLOW + f'CAUGHT ERROR: {e}' + Fore.RESET)
            finally:
                print('----------------------------------------')



import zipfile
import shutil
import re
import sys
from pathlib import Path
from colorama import Fore

def extract_zip(zip_file, dir):
    temp_dir = dir.joinpath('temp')
    temp_dir.mkdir(exist_ok=True)
    
    with zipfile.ZipFile(dir.joinpath(zip_file.name), 'r') as zip:
        zip.extractall(temp_dir)

    submission = extract_py(temp_dir, True)

    if submission is None:
        motherfucking_subdirectories = list(temp_dir.glob('*/'))
        
        if len(motherfucking_subdirectories) == 1:
            jesus_christ = list(motherfucking_subdirectories[0].glob('*.py'))
        
            if len(jesus_christ) == 1 and jesus_christ[0].suffix == '.py':
                return jesus_christ[0]
            else:
                # THIS IS WHERE I DRAW THE LINE
                return None

    return submission

def extract_py(dir, give_up = False):
    python_files = list(dir.glob('*.py'))
    submission = None

    match len(python_files):
        case 0:
            if not give_up:
                submission = extract_zip(dir)
        case 1:
            if not python_files[0].is_dir():
                submission = python_files[0]
        case _:
            if not python_files[-1].is_dir():
                submission = python_files[-1] 

    return submission

def extract_submissions(target_directory, verbose=False):
    ERROR_STATE = False
    count = 0

    """
        TODO: handle paths with/without windows and unix slashes
    """        

    output_path = Path(f'{"/".join(target_directory.split('\\'))[:-1]}-submissions')
    output_path.mkdir(exist_ok=True)

    pattern = re.compile('A[0-9]{8}_[a-z\\s\\-]+_[a-z]+', re.IGNORECASE) 

    for sub in Path(target_directory).glob('*'):
        if sub.name == 'index.html':
            continue

        result = re.search(pattern, sub.name)

        if result is None:
            print(Fore.RED + f'Filename pattern not matched for {sub}' + Fore.RESET)
            continue

        student = result.group()

        if sub.is_dir():
            submission = extract_py(sub)
        elif sub.suffix == '.py':
            submission = sub 
        elif sub.suffix == '.zip':
            submission = extract_zip(sub, student)

        if submission is None:
            print(Fore.RED + f'ERROR: could not find submission for {student}' + Fore.RESET)
            ERROR_STATE = True

        _from = submission
        _to = f"{output_path.name}/{student.replace(' ', '_')}.py"
        
        try:
            shutil.copy(_from, _to)
        except:
            print(Fore.RED + f'ERROR: could not copy submission for student {student}' + Fore.RESET)
            ERROR_STATE = True
        else:
            count += 1  
    
    if verbose:
        print(Fore.GREEN + f'Output {count} submissions to {output_path}' + Fore.RESET)

    return {
        'e': ERROR_STATE,
        'output_path': output_path
    }


import os
import json
from colorama import Fore
from acit1515.tests.test import Test
from acit1515.helpers import Transcript
from pathlib import Path

class Load_Transcript(Test):
    def __init__(self, submission):
        self.name = f'{__class__.__name__.lower()}()'
        self.submission = submission

    def test0_handles_invalid_json(self):
        handles = True
        dir = Path("/".join(__file__.split('\\')[:-1]))
        
        try:
            empty_path = dir.joinpath('transcripts/Blank-User.json')
            empty = self.submission.load_transcript(empty_path)
        except:
            handles = False 
        else:
            if empty != False:
                handles = False
        
        if not handles:
            print(Fore.RED + f"FAIL: {self.name} does not handle empty/invalid JSON files (-5 marks)" + Fore.RESET)
            return { 'deduction': 5, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }

    def test1_returns(self):
        returns = True
        dir = Path("/".join(__file__.split('\\')[:-1]))
        p = dir.joinpath('transcripts/Test-User.json')

        try:
            updated = self.submission.load_transcript(p)
        except Exception as e:
            returns = False
        else:
            if updated == None:
               returns = False 
        
        if not returns:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-4 marks)" + Fore.RESET)
            return { 'deduction': 4, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_type(self):
        dir = Path("/".join(__file__.split('\\')[:-1]))
        
        valid_path = dir.joinpath('transcripts/Test-User.json')
        invalid_path = dir.joinpath('transcripts/User-Test.json')

        valid = self.submission.load_transcript(valid_path)
        invalid = self.submission.load_transcript(invalid_path)
        
        if not isinstance(valid, Transcript) or invalid != False:
            print(Fore.RED + f"FAIL: {self.name} does not return the correct data type for valid/invalid path (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test3_hardcoded(self):
        dir = Path("/".join(__file__.split('\\')[:-1]))
        
        valid_path = dir.joinpath('transcripts/Test-User.json')
        another_valid_path = dir.joinpath('transcripts/Test-User2.json')
        
        invalid_path = dir.joinpath('transcripts/User-Test.json')

        valid = self.submission.load_transcript(valid_path)
        another_valid = self.submission.load_transcript(another_valid_path)
        invalid = self.submission.load_transcript(invalid_path)

        if (
            isinstance(valid, Transcript) 
            and isinstance(another_valid, Transcript) 
            and valid == invalid
        ) or (
            valid == False 
            and another_valid == False 
            and invalid == False
        ):
            print(Fore.RED + f"FAIL: {self.name} appears to be returning a hardcoded value (-2 marks)" + Fore.RESET)
            return { 'deduction': 2, 'continue': False }
    
        return { 'deduction': 0, 'continue': True }
    
    def test4_valid(self):
        returns = True

        dir = Path("/".join(__file__.split('\\')[:-1]))
        valid_path = dir.joinpath('transcripts/Test-User.json')
        
        try:
            with valid_path.open() as file:
                valid_user = json.load(file)
                valid_user = Transcript(
                    valid_user["firstname"], 
                    valid_user["lastname"], 
                    valid_user["grades"]
                )
        except:
            returns = False
        else:
            valid_result = self.submission.load_transcript(valid_path)
            if valid_result != valid_user:
                returns = False
        
        if not returns:
            print(Fore.RED + f"FAIL: {self.name} does not return a valid Transcript object (-1 mark)" + Fore.RESET)
            return { 'deduction': 1, 'continue': False }

        print(Fore.GREEN + f"PASS: {self.name} returned a valid Transcript object (5 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }


from colorama import Fore
from acit1515.tests.test import Test

"""
    TODO: concatenate and log error strings?
"""

class Validate_Choice(Test):
    def __init__(self, submission):
        self.name = 'validate_choice()'
        self.submission = submission

    def test1_returns(self):
        returns = True

        try:
            choice = self.submission.validate_choice(1)
        except Exception as e:
            print(e)
            returns = False
        else:
            if choice == None:
                returns = False
        
        if not returns:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-4 marks)" + Fore.RESET)
            return { 'deduction': 4, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_int(self):
        choice = self.submission.validate_choice(1)
        if not isinstance(choice, int):
            print(Fore.RED + f"FAIL: {self.name} does not return an int (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test3_valid_accepted(self):
        valid_choice = self.submission.validate_choice(1)

        if not valid_choice == 1:
            print(Fore.RED + f"FAIL: {self.name} did not accept a valid choice (-1 mark)" + Fore.RESET)
            return { 'deduction': 1, 'continue': True }
                 
        return { 'deduction': 0, 'continue': True }
    
    def test4_invalid_rejected(self):
        invalid_choice = self.submission.validate_choice(10)

        if not invalid_choice == 0:
            print(Fore.RED + f"FAIL: {self.name} did not reject an invalid choice (-1 mark)" + Fore.RESET)
            return { 'deduction': 1, 'continue': True }
        
        print(Fore.GREEN + f"PASS: {self.name} produced a valid value (4 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }
    
   


import random
from colorama import Fore
from statistics import mean
from acit1515.tests.test import Test

class Get_Average(Test):
    def __init__(self, submission):
        self.name = 'get_average()'
        self.submission = submission

    def test1_returns(self):
        average = self.submission.get_average([7, 7, 7, 7, 7, 7, 7])
        if average == None:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-4 marks)" + Fore.RESET)
            return { 'deduction': 4, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_number(self):
        average = self.submission.get_average([7, 7, 7, 7, 7, 7, 7])
        if not (isinstance(average, int) or isinstance(average, float)):
            print(Fore.RED + f"FAIL: {self.name} does not return an int/float (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test3_hardcoded(self):
        average1 = self.submission.get_average([7, 7, 7, 7, 7, 7, 7])  
        average2 = self.submission.get_average([8, 8, 8, 8, 8, 8, 8])  
        average3 = self.submission.get_average([9, 9, 9, 9, 9, 9, 9])

        if average1 == average2 and average1 == average3:
                print(Fore.RED + f"FAIL: {self.name} appears to be returning hard coded values (-2 marks)" + Fore.RESET)
                return { 'deduction': 2, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test4_valid(self):
        test_grades = random.sample(range(101), 7)
        average = self.submission.get_average(test_grades)
        if round(average, 2) != round(mean(test_grades), 2):
            print(Fore.RED + f"FAIL: {self.name} does not correctly return the average of a list (-1 marks)" + Fore.RESET)
            return { 'deduction': 1, 'continue': False }

        print(Fore.GREEN + f"PASS: {self.name} produced a valid value (4 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }


from colorama import Fore
from acit1515.tests.test import Test

class Get_Status(Test):
    def __init__(self, submission):
        self.name = 'get_status()'
        self.submission = submission

    def test1_returns(self):
        returns = True

        try:
            status = self.submission.get_status([7, 7, 7, 7, 7, 7, 7])
        except:
            returns = False
        else:
            if status == None:
                returns = False
        
        if not returns:
            print(Fore.RED + f"FAIL: {self.name} does not return a value (-3 marks)" + Fore.RESET)
            return { 'deduction': 3, 'continue': False }

        return { 'deduction': 0, 'continue': True }
    
    def test2_bool(self):
        status = self.submission.get_status([7, 7, 7, 7, 7, 7, 7])
        if not isinstance(status, bool):
            print(Fore.RED + f"FAIL: {self.name} does not return a boolean value (-2 marks)" + Fore.RESET)
            return { 'deduction': 2, 'continue': False } 
        
        return { 'deduction': 0, 'continue': True }
    
    def test3_valid(self):
        status1 = self.submission.get_status([7, 7, 7, 7, 7, 7, 7])
        status2 = self.submission.get_status([90, 90, 90, 90, 90, 90, 90])
        
        if status1 and not status2:
            print(Fore.RED + f"FAIL: {self.name} does not correctly return the status (-1 marks)" + Fore.RESET)
            return { 'deduction': 1, 'continue': False }

        print(Fore.GREEN + f"PASS: {self.name} produced a valid value (3 marks)" + Fore.RESET)
        return { 'deduction': 0, 'continue': True }